%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                      %
% Random Direction Mobility Model                                      %
% Author: Wenbo Wang                                                   %
% Date: 1st Aug. 2015 -- the 1st version                               %
%                                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                      %                  
% note: 1.this script is together with fp_v function. Details of       %
%       configuration is in the configuration file (*.txt)             %
%                                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% close all the figures, clear all the variables in the workspace and clear
% the screen
close all;
clear all;
clc;

% initial
xmax=10;ymax=xmax; % the area of test
noOfFloor=1;
floorHeight=3;
tol=0.2; % fingerprint step
n_dev=sqrt(2); % noise deviation
n_ap=2; % path loss coefficient

v=0.9; % the velocity

ang=0:pi/8:pi/2;

Fs=10; % the sampling rate is 10 Hz
vi=v/Fs;

dist=xmax; % movement distance

step=floor(dist/vi);

% calculate the track

track=cell(length(ang),1);

for n=1:length(ang)
    
    temp1=[0,vi*cos(ang(n))*(1:step)];
    temp2=[0,vi*sin(ang(n))*(1:step)];
    track{n,1}=[temp1;temp2;zeros(1,step+1)];
    
end

clear n temp1 temp2;

% receiver power function
Pr=@(x,x1,PT,n_ap,n) PT-10*n_ap.*log10(sqrt(sum((x-x1).^2)))+n;

pv=cell(length(ang),1);

for n=1:length(ang)
    
    % load
    [position,power,s,AP] = fp_v(xmax,ymax,noOfFloor,floorHeight,tol,n_dev,n_ap);
    
    % form noise database
    noise=randn(1,step+1);
    noise=(noise-mean(noise))./std(noise,1);
    noise=n_dev*noise;
    
    % form AP location matrix
    ap_posi=repmat([xmax/2;ymax/2;floorHeight],1,step+1);
    
    % power for different velocity
    pv{n,1}=Pr(track{n,1},ap_posi,0,n_ap,noise);
    
    temp=track{n,1};
    
    x=position(1:s:end,1);
    y=position(1:s,2);
    [X,Y]=meshgrid(x,y);
    z=reshape(power,s,length(power)/s);
    
    figure;
    [h1,hline1,hline2]=plotyy(temp(1,:),temp(2,:),temp(1,:),pv{n,1});
    xlabel('x [m]');
    ylabel(h1(1),'y [m]');
    ylabel(h1(2),'power level [dBm]');
    legend([hline1,hline2],'the track','the power level');
    figure;
    mesh(X,Y,z);
    xlabel('x [m]');
    ylabel('y [m]');
    zlabel('z [m]');
    h3=colorbar;
    set(get(h3,'title'),'string','Power Level [dBm]');
    
end

figure;
plot((0:step)/Fs,pv{1,1},'-o',(0:step)/Fs,pv{2,1},'-^',(0:step)/Fs,pv{3,1},'-s',(0:step)/Fs,pv{4,1},'-d',(0:step)/Fs,pv{5,1},'--');

xlabel('time [s]');
ylabel('power level [dBm]');
legend(['the angle is ', num2str(ang(1)), ' rad'],['the angle is ', num2str(ang(2)), ' rad'],...
    ['the angle is ', num2str(ang(3)), ' rad'],['the angle is ', num2str(ang(4)), ' rad'],...
    ['the angle is ', num2str(ang(5)), ' rad']);