function AP = apDesign(xmax,ymax,noOfFloor,floorHeight)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                      %
% function to design APs location and power                            %
% Author: Wenbo Wang                                                   %
% Date: 1st Aug. 2015 -- the 1st version                               %
%                                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                      %
% - xmax and ymax is the length and width of this building [m]         %
% - noOfFloor is the number of floors                                  %
% - floorHeight is the height spacing of floors                        %
% - AP, the scecond row is the power and third row is the location     %
%                                                                      %
% here we assume 1 AP every 400 m2 and APs are evenly distributed      %
% assume the APs are deployed on the ceiling                           %
% note: we assume the power of AP followed uniform distribution        %
%                                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% assum the AP coverage is 400 m2
cover=400;

% assume PT power of APs are in the interval of [-45,-30] [dB]
PT_up=-30;
PT_down=-45;

% total amount of APs in the building
noOfAp=ceil(xmax*ymax/cover);
totalAP=noOfAp*noOfFloor;

% the first AP of the first floor, using 4/5 as the referred ratio
temp_ratio=4*sqrt(1/noOfAp)/5;
temp_x=temp_ratio*xmax;
temp_y=temp_ratio*ymax;
AP{3,1}=[temp_x*rand,temp_y*rand,floorHeight];

% assume that in the same plane, the distance should be not smaller than
% max(xmax,ymax)*(1/noOfAp), and in the different plane, the distance
% should be not smaller than floorHeight+1 meters

ref1=max(xmax,ymax)*(1/noOfAp);
ref2=floorHeight+0.5;
ref=max(ref1,ref2);

% the third dimension
z=floorHeight:floorHeight:floorHeight*noOfFloor;
z=repmat(z,noOfAp,1);

% the first floor AP distribution
apLocation=AP{3,1};

for n=2:noOfAp
    
    while true
        
        temp_x=xmax*rand;
        temp_y=ymax*rand;
        temp=[temp_x,temp_y,z(n)];
        temp1=repmat(temp,n-1,1);
        dist_m=sqrt(sum((apLocation-temp1).^2,2));
        
        if dist_m>ref
            AP{3,n}=temp;
            break;
        end
        
    end
    
    apLocation(n,:)=AP{3,n};
    
end

% the second and higher floor AP distribution
for p=2:noOfFloor
    
    count=0;
        
    for pp=(p-1)*noOfAp+1:p*noOfAp
        
        while true
            
            temp_x=xmax*rand;
            temp_y=ymax*rand;
            temp=[temp_x,temp_y,z(pp)];
            temp1=repmat(temp,noOfAp+count,1);
            dist_m=sqrt(sum((apLocation((p-2)*noOfAp+1:pp-1,:)-temp1).^2,2));
            
            if dist_m>ref
                AP{3,pp}=temp;
                break;
            end
            
        end
        
        count=count+1;
        apLocation(pp,:)=AP{3,pp};
            
    end
        
end

power=zeros(totalAP,1);

for q=1:totalAP
    
    AP{1,q}=q;
    AP{2,q}=PT_down+(PT_up-PT_down)*rand+30; % transfer dB into dBm
    power(q,:)=AP{2,q};
    
end
  
% figure of location of APs
scatter3(apLocation(:,1),apLocation(:,2),apLocation(:,3),30,power,'fill');

h=colorbar;
set(get(h),'title','Power [dB]')
xlabel('x [m]');
ylabel('y [m]');
zlabel('z [m]');
% ('ylim',[-45,-20],'ytick',[-45,-40,-35,-30,-25,-20],'yticklabel',...
%     {'-45dBm','-40dBm','-35dBm','-30dBm','-25dBm','-20dBm'})

end