%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                      %
% Boundless Simulation Area Mobility Model                             %
% Author: Wenbo Wang                                                   %
% Date: 1st Aug. 2015 -- the 1st version                               %
%                                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                      %                  
% Fixed distance (2-D model)                                           %
% bug: sometimes the point goes out of the boundary, this problem will %
% be fixed in the later version                                        %
%                                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% close all the figures, clear all the variables in the workspace and clear
% the screen
close all;
clear all;
clc;
path(path,'functions');
path(path, 'classes');
%% variable
x0=3;y0=3;                  % starting point
x_max=5;y_max=5;            % boundary
v_min=0.5;v_max=0.8;        % limitation of velocity
theta_min=-2*pi;theta_max=2*pi; % limitation of direction
N=200;                      % steps of movement

%%{
%% 2-D
v=v_min+(v_max-v_min).*rand(1,N);% form the velocities set
theta=theta_min+(theta_max-theta_min).*rand(1,N);% form the angles set
trace_temp=ones(2,N+1);     % initialize the space for storing track points
trace_temp([1 2],1)=[x0;y0];% define the start point of track

% calculate the velocities in both x- and y-axis directions
temp_cos=cos(theta);
temp_sin=sin(theta);
temp_x=v.*temp_cos;         % x-axis direction
temp_y=v.*temp_sin;         % y-axis direction

% random walk process, in this 'for loop' all the track points are calculated but not transfered into doughnut-shaped area 
for t=1:N
    trace_temp([1 2],t+1)=[x0+sum(temp_x(1:t));y0+sum(temp_y(1:t))];
end

% initialize the container of axe for plotting
hOut = inout.output();      
hOut.figure;

% plotting the framework (the boundary in other words)...
h4=plot([0 x_max],[y_max y_max],'r-','linewidth',1.5);
plot([0 x_max],[0 0],'r-','linewidth',1.5);
plot([0 0],[0 y_max],'r-','linewidth',1.5);
plot([x_max x_max],[0 y_max],'r-','linewidth',1.5);

% the first point of the track in the boundless area
trace(1,1)=x0;
trace(2,1)=y0;

% set a counter, it increases when the user touches the boundary
count=1;

% processing...
for t=1:N
    
    % ### here the first 'if' statement will be explained in details
    
    if trace_temp(1,t+1)>=x_max&&0<trace_temp(2,t+1)<y_max
        %      #                                     #              
        %      #                                     #
        %###################################################
        %      #                                     #  $$
        %###################################################
        %      #                                     #
        %      #                                     #
        %
        % this 'if' statement describes the situation when the point goes
        % out of the boundary and end up in the area marked by '$$' (we
        % assume the '#' is representative for the boundary)
        
        % calculate the intersection point with the boundary x=x_max
        y=((trace_temp(2,t+1)-trace_temp(2,t))/(trace_temp(1,t+1)-trace_temp(1,t)))...
            *x_max+((trace_temp(1,t+1)*trace_temp(2,t)-trace_temp(1,t)*trace_temp(2,t+1))/(trace_temp(1,t+1)-trace_temp(1,t)));
        
        % storing the coordinate of points...
        trace(1,count+1)=x_max;
        trace(2,count+1)=y;
        
        % plotting the line connected the previous point with current
        % point, 'x' represents the user is going in the boundary
        plot([trace(1,count),x_max],[trace(2,count),y],'k-',x_max,y,'bx','linewidth',0.8,'markersize',7);hold on;
        
        % because of the intersection point, the counter increase by 1
        count=count+1;
        
        % calculate all the points coordinates after the current point, the
        % whole points set move left x_max value in this case
        trace_temp(1,t+1:end)=trace_temp(1,t+1:end)-x_max;
        trace_temp(2,t+1:end)=trace_temp(2,t+1:end);
        
        % storing the next point coordinate...
        trace(1,count+1)=0;
        trace(2,count+1)=y;
        
        % because the new point is emerged, the counter increases by 1
        count=count+1;
        
        % storing the next next point into the memory...
        trace(1,count+1)=trace_temp(1,t+1);
        trace(2,count+1)=trace_temp(2,t+1);
        
        % plotting the line connected the next point with the next next
        % poing, the 'o' represents the point is going out from the
        % boundary
        plot(trace(1,count:count+1),trace(2,count:count+1),'k-',0,y,'mo','linewidth',0.8,'markersize',7);hold on;
        
        % because the birth of the next next point, the counter increases
        % by 1
        count=count+1;
        
        % other cases
    elseif trace_temp(1,t+1)>=x_max&&trace_temp(2,t+1)<=0
        trace(1,count+1)=x_max;
        trace(2,count+1)=0;
        plot([trace(1,count),x_max],[trace(2,count),0],'k-',x_max,0,'bx','linewidth',0.8,'markersize',7);hold on;
        count=count+1;
        trace_temp(1,t+1:end)=trace_temp(1,t+1:end)-x_max;
        trace_temp(2,t+1:end)=trace_temp(2,t+1:end)+y_max;
        trace(1,count+1)=0;
        trace(2,count+1)=y_max;
        count=count+1;
        trace(1,count+1)=trace_temp(1,t+1);
        trace(2,count+1)=trace_temp(2,t+1);
        plot(trace(1,count:count+1),trace_temp(2,count:count+1),'k-',0,y_max,'mo','linewidth',0.8,'markersize',7);hold on;
        count=count+1;
    elseif 0<trace_temp(1,t+1)<x_max&&trace_temp(2,t+1)<=0
        x=trace_temp(1,t)-trace_temp(2,t)*(trace_temp(1,t+1)-trace_temp(1,t))/(trace_temp(2,t+1)-trace_temp(2,t));
        trace(1,count+1)=x;
        trace(2,count+1)=0;
        plot([trace(1,count),x],[trace(2,count),0],'k-',x,0,'bx','linewidth',0.8,'markersize',7);hold on;
        count=count+1;
        trace_temp(1,t+1:end)=trace_temp(1,t+1:end);
        trace_temp(2,t+1:end)=trace_temp(2,t+1:end)+y_max;
        trace(1,count+1)=x;
        trace(2,count+1)=y_max;
        count=count+1;
        trace(1,count+1)=trace_temp(1,t+1);
        trace(2,count+1)=trace_temp(2,t+1);
        plot(trace(1,count:count+1),trace(2,count:count+1),'k-',x,y_max,'mo','linewidth',0.8,'markersize',7);hold on;
        count=count+1;
    elseif trace_temp(1,t+1)<=0&&trace_temp(2,t+1)<=0
        trace(1,count+1)=0;
        trace(2,count+1)=0;
        plot([trace(1,count),0],[trace(2,count),0],'k-',0,0,'bx','linewidth',0.8,'markersize',7);hold on;
        count=count+1;
        trace_temp(1,t+1:end)=trace_temp(1,t+1:end)+x_max;
        trace_temp(2,t+1:end)=trace_temp(2,t+1:end)+y_max;
        trace(1,count+1)=x_max;
        trace(2,count+1)=y_max;
        count=count+1;
        trace(1,count+1)=trace_temp(1,t+1);
        trace(2,count+1)=trace_temp(2,t+1);
        plot(trace(1,count:count+1),trace(2,count:count+1),'k-',x_max,y_max,'mo','linewidth',0.8,'markersize',7);hold on;
        count=count+1;
    elseif trace_temp(1,t+1)<=0&&0<trace_temp(2,t+1)<y_max
        y=(trace_temp(1,t+1)*trace_temp(2,t)-trace_temp(1,t)*trace_temp(2,t+1))/(trace_temp(1,t+1)-trace_temp(1,t));
        trace(1,count+1)=0;
        trace(2,count+1)=y;
        plot([trace(1,count),0],[trace(2,count),y],'k-',0,y,'bx','linewidth',0.8,'markersize',7);hold on;
        count=count+1;
        trace_temp(1,t+1:end)=trace_temp(1,t+1:end)+x_max;
        trace_temp(2,t+1:end)=trace_temp(2,t+1:end);
        trace(1,count+1)=x_max;
        trace(2,count+1)=y;
        count=count+1;
        trace(1,count+1)=trace_temp(1,t+1);
        trace(2,count+1)=trace_temp(2,t+1);
        plot(trace(1,count:count+1),trace(2,count:count+1),'k-',x_max,y,'mo','linewidth',0.8,'markersize',7);hold on;
        count=count+1;
    elseif trace_temp(1,t+1)<=0&&trace_temp(2,t+1)>=y_max
        trace(1,count+1)=0;
        trace(2,count+1)=y_max;
        plot([trace(1,count),0],[trace(2,count),y_max],'k-',0,y_max,'bx','linewidth',0.8,'markersize',7);hold on;
        count=count+1;
        trace_temp(1,t+1:end)=trace_temp(1,t+1:end)+x_max;
        trace_temp(2,t+1:end)=trace_temp(2,t+1:end)-y_max;
        trace(1,count+1)=x_max;
        trace(2,count+1)=0;
        count=count+1;
        trace(1,count+1)=trace_temp(1,t+1);
        trace(2,count+1)=trace_temp(2,t+1);
        plot(trace(1,count:count+1),trace(2,count:count+1),'k-',x_max,0,'mo','linewidth',0.8,'markersize',7);hold on;
        count=count+1;
    elseif 0<trace_temp(1,t+1)<x_max&&trace_temp(2,t+1)>=y_max
        x=trace_temp(1,t)+(y_max-trace_temp(2,t))*(trace_temp(1,t+1)-trace_temp(1,t))/(trace_temp(2,t+1)-trace_temp(2,t));
        trace(1,count+1)=x;
        trace(2,count+1)=y_max;
        plot([trace(1,count),x],[trace(2,count),y_max],'k-',x,y_max,'bx','linewidth',0.8,'markersize',7);hold on;
        count=count+1;
        trace_temp(1,t+1:end)=trace_temp(1,t+1:end);
        trace_temp(2,t+1:end)=trace_temp(2,t+1:end)-y_max;
        trace(1,count+1)=x;
        trace(2,count+1)=0;
        count=count+1;
        trace(1,count+1)=trace_temp(1,t+1);
        trace(2,count+1)=trace_temp(2,t+1);
        plot(trace(1,count:count+1),trace(2,count:count+1),'k-',x,0,'mo','linewidth',0.8,'markersize',7);hold on;
        count=count+1;
    elseif trace_temp(1,t+1)>=x_max&&trace_temp(2,t+1)>=y_max
        trace(1,count+1)=x_max;
        trace(2,count+1)=y_max;
        plot([trace(1,count),x_max],[trace(2,count),y_max],'k-',x_max,y_max,'bx','linewidth',0.8,'markersize',7);hold on;
        count=count+1;
        trace_temp(1,t+1:end)=trace_temp(1,t+1:end)-x_max;
        trace_temp(2,t+1:end)=trace_temp(2,t+1:end)-y_max;
        trace(1,count+1)=0;
        trace(1,count+1)=0;
        count=count+1;
        plot(temp(1,count:count+1),trace(2,count:count+1),'k-',0,0,'mo','linewidth',0.8,'markersize',7);hold on;
        count=count+1;
    else
        trace(1,count+1)=trace_temp(1,t);
        trace(2,count+1)=trace_temp(2,t);
        h3=plot(trace(1,count:count+1),trace(2,count:count+1),'k-','linewidth',0.8,'markersize',7);
        count=count+1;
        hold on;grid on;
    end
    
    % restrict the axis range
    axis([-1 x_max+1 -1 y_max+1]);
    
    % open an animate
    drawnow;
end

% plotting the starting point and the ending point...
h1=plot(trace(1,1),trace(2,1),'go','markersize',10,'linewidth',1.5);
h2=plot(trace(1,end),trace(2,end),'ro','markersize',10,'linewidth',1.5);
legend([h1,h2,h3,h4],'starting point','terminal point','track','boundary');

xlabel('x [m]');
ylabel('y [m]');
axis square;
grid on;
hold off;
%}