%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                      %
% Boundless Simulation Area 3-D Model                                  %
% Author: Wenbo Wang                                                   %
% Date: 1st Aug. 2015 -- the 1st version                               %
%                                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% close all the figures, clear all the variables in the workspace and clear
% the screen
close all;
clear all;
clc;
path(path,'functions');
path(path, 'classes');

x0=3;y0=6; %starting point
x_max=5;y_max=5;  %boundary
v_min=0.8;v_max=1.4;    %limitation of velocity
theta_min=-pi;theta_max=pi; %limitation of direction
n=30;
N=1000; %steps of movement

%******************************************
%        boundless concept in 3-D
%******************************************

R=x_max/(2*pi);
r=y_max/(2*pi);
u=0:2*pi/n:2*pi;
v=0:2*pi/n:2*pi;
[u,v]=meshgrid(u,v);
x=(R+r.*cos(u)).*cos(v);
y=(R+r.*cos(u)).*sin(v);
z=r.*sin(u);

figure;
mesh(x,y,z);
%set(gcf,'color',[0 0 0]);
colormap([0 0 0]);
xlabel('x [m]');
ylabel('y [m]');
zlabel('z [m]');
axis equal;grid on;
%view(3);
%title('Boundless Simulation Area');

%{
%******************************************
%       boundless simulation in 2-D
%******************************************

v=v_min+(v_max-v_min).*rand(1,N);
theta=theta_min+(theta_max-theta_min).*rand(1,N);
trace=ones(2,N+1);  %define the start point of trace
trace([1 2],1)=[x0;y0];
temp_cos=cos(theta);
temp_sin=sin(theta);
temp_x=v.*temp_cos;
temp_y=v.*temp_sin;
for t=1:N
    trace([1 2],t+1)=[x0+sum(temp_x(1:t));y0+sum(temp_y(1:t))];
end
figure(2);
modobj = VideoWriter('boundless','mpeg-4');
open(modobj);
count=1;
for t=1:N
    plot(trace(1,count:t),trace(2,count:t));grid on;hold on;
    axis([0 15 0 15]);
    if trace(1,t+1)>=x_max&0<trace(2,t+1)<y_max
        temp1=((trace(2,t+1)-trace(2,t))/(trace(1,t+1)-trace(1,t)))*x_max+((trace(1,t+1)*trace(2,t)-trace(1,t)*trace(2,t+1))/(trace(1,t+1)-trace(1,t)));
        temp2=temp1;
        plot([trace(1,t),x_max],[trace(2,t),temp1],'-',x_max,temp1,'x');hold on;
        trace(1,t+1:end)=trace(1,t+1:end)-x_max;
        trace(2,t+1:end)=trace(2,t+1:end);
        count=t+1;
        plot([0,trace(1,t+1)],[temp2,trace(2,t+1)],'-',0,temp2,'o');hold on;
    elseif trace(1,t+1)>=x_max&trace(2,t+1)<=0
        plot([trace(1,t),x_max],[trace(2,t),0],'-',x_max,0,'x');hold on;
        trace(1,t+1:end)=trace(1,t+1:end)-x_max;
        trace(2,t+1:end)=trace(2,t+1:end)+y_max;
        count=t+1;
        plot([0,trace(1,t+1)],[y_max,trace(2,t+1)],'-',0,y_max,'o');hold on;
    elseif 0<trace(1,t+1)<x_max&trace(2,t+1)<=0
        temp1=((trace(1,t)*trace(2,t+1)-(trace(1,t+1)*trace(2,t))/(trace(1,t+1)-trace(1,t))))/((trace(2,t+1)-trace(2,t))/(trace(1,t+1)-trace(1,t)));
        temp2=temp1;
        plot([trace(1,t),temp1],[trace(2,t),0],'-',temp1,0,'x');hold on;
        trace(1,t+1:end)=trace(1,t+1:end);
        trace(2,t+1:end)=trace(2,t+1:end)+y_max;
        count=t+1;
        plot([temp2,trace(1,t+1)],[y_max,trace(2,t+1)],'-',temp2,y_max,'o');hold on;
    elseif trace(1,t+1)<=0&trace(2,t+1)<=0
        plot([trace(1,t),0],[trace(2,t),0],'-',0,0,'x');hold on;
        trace(1,t+1:end)=trace(1,t+1:end)+x_max;
        trace(2,t+1:end)=trace(2,t+1:end)+y_max;
        count=t+1;
        plot([x_max,trace(1,t+1)],[y_max,trace(2,t+1)],'-',x_max,y_max,'o');hold on;
    elseif trace(1,t+1)<=0&0<trace(2,t+1)<y_max
        temp1=(trace(1,t+1)*trace(2,t)-trace(1,t)*trace(2,t+1))/(trace(1,t+1)-trace(1,t));
        temp2=temp1;
        plot([trace(1,t),0],[trace(2,t),temp1],'-',0,temp1,'x');hold on;
        trace(1,t+1:end)=trace(1,t+1:end)+x_max;
        trace(2,t+1:end)=trace(2,t+1:end);
        count=t+1;
        plot([x_max,trace(1,t+1)],[temp2,trace(2,t+1)],'-',x_max,temp2,'o');hold on;
    elseif trace(1,t+1)<=0&trace(2,t+1)>=y_max
        plot(trace(1,count:t),trace(2,count:t));hold on;
        plot([trace(1,t),0],[trace(2,t),y_max],'-',0,y_max,'x');hold on;
        trace(1,t+1:end)=trace(1,t+1:end)+x_max;
        trace(2,t+1:end)=trace(2,t+1:end)-y_max;
        count=t+1;
        plot([x_max,trace(1,t+1)],[0,trace(2,t+1)],'-',x_max,0,'o');hold on;
    elseif 0<trace(1,t+1)<x_max&trace(2,t+1)>=y_max
        temp1=(y_max+(trace(1,t)*trace(2,t+1)-(trace(1,t+1)*trace(2,t)))/(trace(1,t+1)-trace(1,t)))/((trace(2,t+1)-trace(2,t))/(trace(1,t+1)-trace(1,t)));
        temp2=temp1;
        plot([trace(1,t),temp1],[trace(2,t),y_max],'-',temp1,y_max,'x');hold on;
        trace(1,t+1:end)=trace(1,t+1:end);
        trace(2,t+1:end)=trace(2,t+1:end)-y_max;
        count=t+1;
        plot([temp2,trace(1,t+1)],[0,trace(2,t+1)],'-',temp2,0,'o');hold on;
    elseif trace(1,t+1)>=x_max&trace(2,t+1)>=y_max
        plot([trace(1,t),x_max],[trace(2,t),y_max],'-',x_max,y_max,'x');hold on;
        trace(1,t+1:end)=trace(1,t+1:end)-x_max;
        trace(2,t+1:end)=trace(2,t+1:end)-y_max;
        count=t+1;
        plot([0,trace(1,t+1)],[0,trace(2,t+1)],'-',0,0,'o');hold on;
    end
    frame=getframe;
    writeVideo(modobj, frame);
end
close(modobj);
%}