%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                      %
% Random Waypoint Mobility Model                                       %
% Author: Wenbo Wang                                                   %
% Date: 1st Aug. 2015 -- the 1st version                               %
%                                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                      %                  
% Fixed distance & Fixed time (2-D model)                              %
%                                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% close all the figures, clear all the variables in the workspace and clear
% the screen
close all;
clear all;
clc;
path(path,'functions');
path(path, 'classes');


%% variables
%vmax=1.5;              % vmax is the maximum velocity
vmax=8;
k=1;
N=100;  %number of steps or number of fixed time interval,we assume this time interval is k=1 second
x0=150;y0=150;z0=0;     % the coordinate of starting point
rmin=-pi;rmax=pi;   	% the range of heading directions
v=vmax*rand(1,N);       % form the velocities set
r=rmin+(rmax-rmin).*rand(1,N);% form the directions set

% zmin=0;zmax=30;%assume a building 30 meters high

hOut = inout.output();  % initialize the container of axe for plotting

%%{
%% 2-D
%% fixed time 2-D
hOut.figure;            % create an empty axe
trace1=zeros(2,N+1);    % initialize the space for storing the track points
trace1(1,1)=x0;         % the x-axis value of the first point
trace1(2,1)=y0;         % the y-axis value of the first point

% #### if you want to film a video, the following 2 lines should release
% from comment, together with "%frame=getframe; %writeVideo(modobj, frame);%close(modobj);"

%modobj = VideoWriter('fixed time','mpeg-4');
%open(modobj);

pmin=2;pmax=10;         % the maximum and minimum value of the pause time [unit: s]
p=pmin+(pmax-pmin)*rand(1,N-1);% form the pause time set

% plotting the starting point...
h1=plot(150,150,'go','markersize',10,'linewidth',1.5);hold on;grid on;
axis([100,200,100,200]);

% processing...
for n=1:N
    
    % calculate the coordinate of the n+1 point from the n point
    trace1(1,n+1)=trace1(1,n)+v(n)*cos(r(n))*k;
    trace1(2,n+1)=trace1(2,n)+v(n)*sin(r(n))*k;
    
    % plotting one jump and giving the n+1 point a marker
    h5=plot([trace1(1,n),trace1(1,n+1)],[trace1(2,n),trace1(2,n+1)],'k-','linewidth',0.6);
    %plot(trace1(1,n+1),trace1(2,n+1),'k.','markersize',8);
    
    % drawnow is an animate command
    drawnow;
    
    %h=plot(trace(1,n+1),trace(2,n+1),'ro','markersize',10);
    %frame=getframe;
    %writeVideo(modobj, frame);
    %set(h,'marker','none');
end

% plotting the ending point...
h2=plot(trace1(1,N+1),trace1(2,N+1),'ro','markersize',10,'linewidth',1.5);

% plotting the pause time, the pause is indicated by the colour of the
% marker
scatter(trace1(1,2:N),trace1(2,2:N),20,p,'filled');
xlabel('x [m]');
ylabel('y [m]');
%text(150,150,'start point');
%text(trace1(1,N+1),trace1(2,N+1),'end point');
legend([h1,h2,h5],'starting point','terminal point','track');
%xlabel(['Time interval is ',num2str(k),' s']);
%title(['constant time',' total duration is ',num2str(N*k),' s']);
axis square;
h=colorbar;             % open the colourbar to indicate the value of the pause time markers
set(get(h,'Title'),'string','pause time [s]');% name a title for the colourbar
hold off;
%close(modobj);

%% fixed distance 2-D
d=3;                    % displacement per step
hOut.figure;            % create an empty axe
trace=zeros(2,N+1);     % initialize the space for storing the track points
trace(1,1)=x0;          % the coordinate of starting point
trace(2,1)=y0;

%modobj = VideoWriter('fixed distance','mpeg-4');
%open(modobj);

% plotting the starting point...
h3=plot(x0,y0,'go','markersize',10,'linewidth',1.5);hold on;grid on;
axis([100,200,100,200]);

% processing...
for n=1:N
    
    % calculate the coordinate of the n+1 point from the n point
    trace(1,n+1)=trace(1,n)+d*cos(r(n));
    trace(2,n+1)=trace(2,n)+d*sin(r(n));

    % plotting one jump and giving the n+1 point a marker...
    h6=plot([trace(1,n),trace(1,n+1)],[trace(2,n),trace(2,n+1)],'k-','linewidth',0.6);
    %plot(trace(1,n+1),trace(2,n+1),'k.','markersize',8);
    
    % drawnow is an animate command
    drawnow;
    
    %h=plot(trace(1,n+1),trace(2,n+1),'ro','markersize',10);
    %frame=getframe;
    %writeVideo(modobj, frame);
    %set(h,'marker','none');
end

% plotting the ending point...
h4=plot(trace(1,N+1),trace(2,N+1),'ro','markersize',10,'linewidth',1.5);

% plotting the pause time, the pause is indicated by the colour of the
% marker
scatter(trace(1,2:N),trace(2,2:N),20,p,'filled');
xlabel('x [m]');
ylabel('y [m]');
legend([h3,h4,h6],'starting point','terminal point','track');
%xlabel(['Each step is ',num2str(d),' m']);
%title(['constant distance',' total steps is ',num2str(N),' steps']);
axis square;
hold off;
h5=colorbar;% open the colourbar to indicate the value of the pause time markers
set(get(h5,'Title'),'string','pause time [s]');% name a title for the colourbar
%close(modobj);
%}

%{
%% 3-D z value

trace2(3,:)=thirdDi(N,1,z0,zmin,zmax);
trace3=zeros(3,N+1);

%% fixed time 3-D
figure;
trace2(1,1)=x0;
trace2(2,1)=y0;
for n=1:N
    trace2(1,n+1)=trace2(1,n)+v(n)*cos(r(n))*k;
    trace2(2,n+1)=trace2(2,n)+v(n)*sin(r(n))*k;
    plot3(x0,y0,z0,'k.','markersize',15);hold on;grid on;
    axis([0,300,0,300,zmin,zmax]);
    plot3([trace2(1,n),trace2(1,n+1)],[trace2(2,n),trace2(2,n+1)],[trace2(3,n),trace2(3,n+1)],'k-','linewidth',0.6);
    plot3(trace2(1,n+1),trace2(2,n+1),trace2(3,n+1),'k.','markersize',8);
    drawnow;
end
plot3(trace2(1,N+1),trace2(2,N+1),trace2(3,N+1),'ro','markersize',10);
title(['constant time',' total duration is ',num2str(N*k),' s']);
hold off;


%% fixed distance 3-D
figure;
d=1.5*rand(1,N);
trace3(1,1)=x0;
trace3(2,1)=y0;
for n=1:N
    trace3(1,n+1)=trace3(1,n)+d(n).*cos(r(n));
    trace3(2,n+1)=trace3(2,n)+d(n).*sin(r(n));
    plot3(x0,y0,z0,'k.','markersize',15);hold on;grid on;
    axis([0,300,0,300,zmin,zmax]);
    plot3([trace3(1,n),trace3(1,n+1)],[trace3(2,n),trace3(2,n+1)],[trace2(3,n),trace2(3,n+1)],'k-','linewidth',0.6);
    plot3(trace3(1,n+1),trace3(2,n+1),trace2(3,n+1),'k.','markersize',8);
    drawnow;
end
plot3(trace3(1,N+1),trace3(2,N+1),trace2(3,N+1),'ro','markersize',10);
title(['constant distance',' total steps is ',num2str(N),' steps']);
hold off;
%}